﻿using BPA.SAAS.Manage.Application.Device.Dtos.DeviceVesion;
using BPA.SAAS.Manage.Application.Device.Interface;
using BPA.SAAS.Manage.Comm.Const;
using BPA.SAAS.Manage.Comm.Enum;
using BPA.SAAS.Manage.Core.Base;
using BPA.SAAS.Manage.Core.Device;
using BPA.SAAS.Manage.Core.Product;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.Device.Services
{
    public class DeviceVesionService: IDeviceVesionService, ITransient
    {
        ISqlSugarClient _db;
        public DeviceVesionService(ISqlSugarClient db)
        {
            _db=db;
        }
        /// <summary>
        /// 分页查询
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<PageUtil> GetDeviceVesionPageAsync(DeviceVesionQueryInputDto inputDto)
        {
            var total = new RefAsync<int>();
            var data = await _db.Queryable<BPA_ProductVesion,BPA_Product>((a, b) => new JoinQueryInfos(JoinType.Left, b.Id == a.ProductId))
                .WhereIF(!string.IsNullOrWhiteSpace(inputDto.ProductId), x => x.ProductId.Contains(inputDto.ProductId))
                .OrderBy(a => a.CreateAt, OrderByType.Desc)
                .Select((a,b)=>new DeviceVesionModel()
                {
                    Id=a.Id.SelectAll(),
                    ProductName=b.Name,
                    ProductCode=b.Code,
                })
                .ToPageListAsync(inputDto.Current, inputDto.PageSize, total);

            return new PageUtil()
            {
                Data = data,
                Total = total

            };
        }

        /// <summary>
        /// 新增
        /// </summary>
        /// <returns></returns>
        
        public async Task<bool> AddDeviceVesionAsync(DeviceVesionBaseDto inputDto)
        {
            var res = await _db.Insertable(new BPA_ProductVesion
            {
                Vesion = inputDto.Vesion,
                ProductId = inputDto.ProductId,
                TemplatePath = inputDto.TemplatePath,
                Status = CommonStatus.ENABLE
            }).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
            return res > 0;
        }
        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="inputList"></param>
        /// <returns></returns>
        public async Task<bool> DelDeviceVesionAsync(List<string> inputList)
        {
            var data = await _db.Queryable<BPA_DeviceInfo>().Where(x => inputList.Contains(x.ProductVersionId)).ToListAsync();
            if (data.Count > 0)
            {
                throw Oops.Oh("当前设备类型已使用，无法删除");
            }
            var datas = await _db.Queryable<BPA_ProductVesion>()
                .Where(x => inputList.Contains(x.Id))
                .ToListAsync();
            _db.Deleteable(datas).ExecuteCommand();
            return true;
        }
        /// <summary>
        /// 修改
        /// </summary>
        /// <returns></returns>
        public async Task<bool> UpdateDeviceVesionAsync(DeviceVesionBaseDto inputDto)
        {
            var data = await _db.Queryable<BPA_ProductVesion>().Where(x => x.Id == inputDto.Id).FirstAsync();

            if (data != null)
            {
                var res = _db.Updateable<BPA_ProductVesion>().SetColumns(t => t.Vesion == inputDto.Vesion)
                     .SetColumns(t => t.TemplatePath == inputDto.TemplatePath)
                     .SetColumns(t => t.ProductId == inputDto.ProductId)
                     .SetColumns(t => t.Status == (CommonStatus)inputDto.Status).Where(t => t.Id == inputDto.Id)
                     .ExecuteCommandHasChange();

                return res;
            }

            return false;
        }
        /// <summary>
        /// 更新状态
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<bool> UpdateDeviceVesionSatatus(DeviceVesionSatatusDto inputDto)
        {
            var data = await _db.Queryable<BPA_ProductVesion>().Where(x => x.Id == inputDto.Id).FirstAsync();

            if (data != null)
            {
                var res = _db.Updateable<BPA_ProductVesion>()
                     .SetColumns(t => t.Status == (CommonStatus)inputDto.Status).Where(t => t.Id == inputDto.Id)
                     .ExecuteCommandHasChange();

                return res;
            }

            return false;
        }
        /// <summary>
        /// 根据产品查询版本
        /// </summary>
        /// <param name="Code"></param>
        /// <returns></returns>
        public async Task<List<BPA_ProductVesion>> GetDeviceVesion(string ProductId)
        {
            var resEntity =await _db.Queryable<BPA_ProductVesion>().Where(a => a.ProductId == ProductId && a.Status==0).Select(a => new BPA_ProductVesion() { Id = a.Id.SelectAll() }).ToListAsync();
            return resEntity;
        }
        /// <summary>
        /// 查询版本列表
        /// </summary>
        /// <returns></returns>
        public async Task<List<BPA_ProductVesion>> GetDeviceVesionList()
        {
            var resEntity =await _db.Queryable<BPA_ProductVesion>().Select(a => new BPA_ProductVesion() { Id = a.Id.SelectAll() }).ToListAsync();
            return resEntity;
        }
    }
}
